// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

part of ducene;

class IndexReader {

  final IndexDirectory _directory;

  IndexReader(this._directory);

  Map<String,Map<String,Set<int>>> _indexes;
  Map<String,Map<int,List<String>>> _data;

  void read() {
    _indexes = _directory.inputIndex();
    _data = _directory.inputData();
  }

  int docFreq(String field, String text) => _indexes[field][text].length;
  int maxDoc() => _data['id'].length;

  Document document(int docId, {Iterable<String> fieldsToLoad}) {
    var fields = (fieldsToLoad != null) ? fieldsToLoad : _data.keys;
    Document ret = new Document();
    for (String field in fields) {
      var storedData = (_data[field] != null) ? _data[field] : null;
      var text = (storedData != null) ? storedData[docId] : null;
      ret.add(new Field(field, text, null, null));
    }
    return ret;
  }

  Map<String,Set<int>> getTerms(String field) => _indexes[field];
  Map<int,List<String>> getData(String field) => _data[field];

}

class DirectoryReader {
  static IndexReader open(IndexDirectory directory) => new IndexReader(directory)..read();
}