// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

part of ducene;

class BoolQuery {
  final Op _op;
  BoolQuery(this._op);
  List<TermQuery> _termQueries = [];
  List<Query> _filters = [];
  BoolQuery addQuery(TermQuery tQuery) {
    _termQueries.add(tQuery);
    return this;
  }
  BoolQuery addFilter(Query query) {
    _filters.add(query);
    return this;
  }
  static Analyzer defaultAnalyzer = new KeywordAnalyzer();
  BoolQuery append(String field, String text,
      {Analyzer analyzer, double boost: 1.00}) {
    Analyzer ana = (analyzer == null)? defaultAnalyzer : analyzer;
    for (var token in ana.getTokens(text)) {
      addQuery(new TermQuery(new Term(field, token, boost: boost)));
    }
    return this;
  }
  List<TermQuery> get termQueries => _termQueries;
  List<Query> get filters => _filters;
  Op get op => _op;
}
enum Op {
  and, or
}
abstract class Query {
  Set<int> matchDocIds(IndexSearcher searcher);
}
class TermQuery implements Query {
  final Term _term;
  TermQuery(this._term);
  Term get term => _term;
  Set<int> matchDocIds(IndexSearcher searcher) {
    var terms = searcher.reader.getTerms(_term.field);
    if (terms == null) return new Set();
    return terms[_term.text];
  }
}
class Term {
  final String field;
  final String text;
  double boost;
  Term(this.field, this.text, {double boost: 1.00}) {
    this.boost = boost;
  }
}
class RangeQuery implements Query {
  final String field;
  final double lower;
  final double upper;
  RangeQuery(this.field, this.lower, this.upper);
  Set<int> matchDocIds(IndexSearcher searcher) {
    Set<int> ret = new Set();
    Map<String,Set<int>> terms = searcher.reader.getTerms(field);
    if (terms == null) return ret;
    bool isFirst = true;
    for (String term in terms.keys) {
      double value = double.parse(term);
      Set<int> docIds = new Set();
      if (lower <= value && value <= upper) {
        docIds = terms[term];
      }
      if (isFirst) {
        ret = docIds;
        isFirst = false;
        continue;
      }
      ret = IndexSearcher.unionDocSet(Op.or, ret, docIds);
    }
    return ret;
  }
}